//
//  ImageView.swift
//  Sense2
//
//  Created by Daniel Hauagge on 2/5/16.
//  Copyright © 2016 Daniel Hauagge. All rights reserved.
//

import UIKit

protocol ImageViewDelegate: class {
    func touchBegan(withForce force: CGFloat);
    func touchMoved(withForce force: CGFloat);
    func touchEnded();
}

class ImageView: UIView {
    
    var delegate : ImageViewDelegate?
    var image : UIImage?
    
    var cursorPosition = CGPoint(x: 100, y: 100) {
        didSet {
            // This function is always called after an assignment to cursorPosition.
            // Swift offers other related methods, called Property Observers, 
            // like get, set, and willSet. For more, see 
            // https://developer.apple.com/library/ios/documentation/Swift/Conceptual/Swift_Programming_Language/Properties.html
            self.setNeedsDisplay()
        }
    }
    var cursorRadius = CGFloat(5.0)
    var showCursor = false

    
    // MARK: Initialization
    override init(frame : CGRect) {
        super.init(frame : frame)
        
        resetImage() //CGSizeMake(frame.size.width, frame.size.height))
    }
    
    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        //fatalError("init(coder:) has not been implemented")
        resetImage()
    }
    
    func resetImage() {
        let size = CGSizeMake(self.frame.size.width, self.frame.size.height)
        UIGraphicsBeginImageContextWithOptions(size, true, 0);
        UIColor.whiteColor().setFill()
        UIRectFill(CGRectMake(0, 0, size.width, size.height))
        image = UIGraphicsGetImageFromCurrentImageContext()
        UIGraphicsEndImageContext()
    }
    
    
    // MARK: Touch related
    override func touchesBegan(touches: Set<UITouch>, withEvent event: UIEvent?) {
        if let touch = touches.first {
            delegate?.touchBegan(withForce:touch.force)
        }
    }

    override func touchesMoved(touches: Set<UITouch>, withEvent event: UIEvent?) {
        if let touch = touches.first {
            delegate?.touchMoved(withForce:touch.force)
        }
    }
    
    override func touchesEnded(touches: Set<UITouch>, withEvent event: UIEvent?) {
        delegate?.touchEnded()
    }
    
    override func touchesCancelled(touches: Set<UITouch>?, withEvent event: UIEvent?) {
        delegate?.touchEnded()
    }
    
    
    // MARK: Drawing code
    override func drawRect(rect: CGRect) {
        
        print("cursor size: \(cursorRadius)")
        
        super.drawRect(rect)
        image?.drawInRect(rect)
        
        let context = UIGraphicsGetCurrentContext()
        
        let origin = CGPointMake(
            rect.origin.x + rect.size.width/2.0,
            rect.origin.y + rect.size.height/2.0
        )

        let circleRect = CGRectMake(
            cursorPosition.x - cursorRadius + origin.x,
            cursorPosition.y - cursorRadius + origin.y,
            2 * cursorRadius, 2 * cursorRadius
        )

        CGContextAddEllipseInRect(context, circleRect)

        // Set line attributes
        CGContextSetLineWidth(context, 2.0)
        CGContextSetRGBFillColor(context, 0.85, 0.5, 0.5, 0.5)
        
        CGContextStrokePath(context)
    }
}
